@@ -16,7 +16,9 @@ from fastapi.responses import FileResponse
 
 from open_webui.config import CACHE_DIR
 from open_webui.constants import ERROR_MESSAGES
-from open_webui.env import ENABLE_FORWARD_USER_INFO_HEADERS, SRC_LOG_LEVELS
+from open_webui.env import ENABLE_FORWARD_USER_INFO_HEADERS
+
+from open_webui.models.chats import Chats
 from open_webui.routers.files import upload_file_handler, get_file_content_by_id
 from open_webui.utils.auth import get_admin_user, get_verified_user
 from open_webui.utils.headers import include_user_info_headers
@@ -31,7 +33,6 @@ from open_webui.utils.images.comfyui import (
 from pydantic import BaseModel
 
 log = logging.getLogger(__name__)
-log.setLevel(SRC_LOG_LEVELS["IMAGES"])
 
 IMAGE_CACHE_DIR = CACHE_DIR / "image" / "generations"
 IMAGE_CACHE_DIR.mkdir(parents=True, exist_ok=True)
@@ -44,18 +45,23 @@ def set_image_model(request: Request, model: str):
     request.app.state.config.IMAGE_GENERATION_MODEL = model
     if request.app.state.config.IMAGE_GENERATION_ENGINE in ["", "automatic1111"]:
         api_auth = get_automatic1111_api_auth(request)
-        r = requests.get(
-            url=f"{request.app.state.config.AUTOMATIC1111_BASE_URL}/sdapi/v1/options",
-            headers={"authorization": api_auth},
-        )
-        options = r.json()
-        if model != options["sd_model_checkpoint"]:
-            options["sd_model_checkpoint"] = model
-            r = requests.post(
+
+        try:
+            r = requests.get(
                 url=f"{request.app.state.config.AUTOMATIC1111_BASE_URL}/sdapi/v1/options",
-                json=options,
                 headers={"authorization": api_auth},
             )
+            options = r.json()
+            if model != options["sd_model_checkpoint"]:
+                options["sd_model_checkpoint"] = model
+                r = requests.post(
+                    url=f"{request.app.state.config.AUTOMATIC1111_BASE_URL}/sdapi/v1/options",
+                    json=options,
+                    headers={"authorization": api_auth},
+                )
+        except Exception as e:
+            log.debug(f"{e}")
+
     return request.app.state.config.IMAGE_GENERATION_MODEL
 
 
@@ -106,9 +112,10 @@ class ImagesConfig(BaseModel):
     IMAGES_OPENAI_API_BASE_URL: str
     IMAGES_OPENAI_API_KEY: str
     IMAGES_OPENAI_API_VERSION: str
+    IMAGES_OPENAI_API_PARAMS: Optional[dict | str]
 
     AUTOMATIC1111_BASE_URL: str
-    AUTOMATIC1111_API_AUTH: str
+    AUTOMATIC1111_API_AUTH: Optional[dict | str]
     AUTOMATIC1111_PARAMS: Optional[dict | str]
 
     COMFYUI_BASE_URL: str
@@ -120,6 +127,7 @@ class ImagesConfig(BaseModel):
     IMAGES_GEMINI_API_KEY: str
     IMAGES_GEMINI_ENDPOINT_METHOD: str
 
+    ENABLE_IMAGE_EDIT: bool
     IMAGE_EDIT_ENGINE: str
     IMAGE_EDIT_MODEL: str
     IMAGE_EDIT_SIZE: Optional[str]
@@ -147,6 +155,7 @@ async def get_config(request: Request, user=Depends(get_admin_user)):
         "IMAGES_OPENAI_API_BASE_URL": request.app.state.config.IMAGES_OPENAI_API_BASE_URL,
         "IMAGES_OPENAI_API_KEY": request.app.state.config.IMAGES_OPENAI_API_KEY,
         "IMAGES_OPENAI_API_VERSION": request.app.state.config.IMAGES_OPENAI_API_VERSION,
+        "IMAGES_OPENAI_API_PARAMS": request.app.state.config.IMAGES_OPENAI_API_PARAMS,
         "AUTOMATIC1111_BASE_URL": request.app.state.config.AUTOMATIC1111_BASE_URL,
         "AUTOMATIC1111_API_AUTH": request.app.state.config.AUTOMATIC1111_API_AUTH,
         "AUTOMATIC1111_PARAMS": request.app.state.config.AUTOMATIC1111_PARAMS,
@@ -157,6 +166,7 @@ async def get_config(request: Request, user=Depends(get_admin_user)):
         "IMAGES_GEMINI_API_BASE_URL": request.app.state.config.IMAGES_GEMINI_API_BASE_URL,
         "IMAGES_GEMINI_API_KEY": request.app.state.config.IMAGES_GEMINI_API_KEY,
         "IMAGES_GEMINI_ENDPOINT_METHOD": request.app.state.config.IMAGES_GEMINI_ENDPOINT_METHOD,
+        "ENABLE_IMAGE_EDIT": request.app.state.config.ENABLE_IMAGE_EDIT,
         "IMAGE_EDIT_ENGINE": request.app.state.config.IMAGE_EDIT_ENGINE,
         "IMAGE_EDIT_MODEL": request.app.state.config.IMAGE_EDIT_MODEL,
         "IMAGE_EDIT_SIZE": request.app.state.config.IMAGE_EDIT_SIZE,
@@ -187,12 +197,12 @@ async def update_config(
     set_image_model(request, form_data.IMAGE_GENERATION_MODEL)
     if (
         form_data.IMAGE_SIZE == "auto"
-        and form_data.IMAGE_GENERATION_MODEL != "gpt-image-1"
+        and not form_data.IMAGE_GENERATION_MODEL.startswith("gpt-image")
     ):
         raise HTTPException(
             status_code=400,
             detail=ERROR_MESSAGES.INCORRECT_FORMAT(
-                "  (auto is only allowed with gpt-image-1)."
+                "  (auto is only allowed with gpt-image models)."
             ),
         )
 
@@ -224,6 +234,9 @@ async def update_config(
     request.app.state.config.IMAGES_OPENAI_API_VERSION = (
         form_data.IMAGES_OPENAI_API_VERSION
     )
+    request.app.state.config.IMAGES_OPENAI_API_PARAMS = (
+        form_data.IMAGES_OPENAI_API_PARAMS
+    )
 
     request.app.state.config.AUTOMATIC1111_BASE_URL = form_data.AUTOMATIC1111_BASE_URL
     request.app.state.config.AUTOMATIC1111_API_AUTH = form_data.AUTOMATIC1111_API_AUTH
@@ -243,15 +256,16 @@ async def update_config(
     )
 
     # Edit Image
+    request.app.state.config.ENABLE_IMAGE_EDIT = form_data.ENABLE_IMAGE_EDIT
     request.app.state.config.IMAGE_EDIT_ENGINE = form_data.IMAGE_EDIT_ENGINE
     request.app.state.config.IMAGE_EDIT_MODEL = form_data.IMAGE_EDIT_MODEL
     request.app.state.config.IMAGE_EDIT_SIZE = form_data.IMAGE_EDIT_SIZE
 
     request.app.state.config.IMAGES_EDIT_OPENAI_API_BASE_URL = (
-        form_data.IMAGES_OPENAI_API_BASE_URL
+        form_data.IMAGES_EDIT_OPENAI_API_BASE_URL
     )
     request.app.state.config.IMAGES_EDIT_OPENAI_API_KEY = (
-        form_data.IMAGES_OPENAI_API_KEY
+        form_data.IMAGES_EDIT_OPENAI_API_KEY
     )
     request.app.state.config.IMAGES_EDIT_OPENAI_API_VERSION = (
         form_data.IMAGES_EDIT_OPENAI_API_VERSION
@@ -287,6 +301,7 @@ async def update_config(
         "IMAGES_OPENAI_API_BASE_URL": request.app.state.config.IMAGES_OPENAI_API_BASE_URL,
         "IMAGES_OPENAI_API_KEY": request.app.state.config.IMAGES_OPENAI_API_KEY,
         "IMAGES_OPENAI_API_VERSION": request.app.state.config.IMAGES_OPENAI_API_VERSION,
+        "IMAGES_OPENAI_API_PARAMS": request.app.state.config.IMAGES_OPENAI_API_PARAMS,
         "AUTOMATIC1111_BASE_URL": request.app.state.config.AUTOMATIC1111_BASE_URL,
         "AUTOMATIC1111_API_AUTH": request.app.state.config.AUTOMATIC1111_API_AUTH,
         "AUTOMATIC1111_PARAMS": request.app.state.config.AUTOMATIC1111_PARAMS,
@@ -297,6 +312,7 @@ async def update_config(
         "IMAGES_GEMINI_API_BASE_URL": request.app.state.config.IMAGES_GEMINI_API_BASE_URL,
         "IMAGES_GEMINI_API_KEY": request.app.state.config.IMAGES_GEMINI_API_KEY,
         "IMAGES_GEMINI_ENDPOINT_METHOD": request.app.state.config.IMAGES_GEMINI_ENDPOINT_METHOD,
+        "ENABLE_IMAGE_EDIT": request.app.state.config.ENABLE_IMAGE_EDIT,
         "IMAGE_EDIT_ENGINE": request.app.state.config.IMAGE_EDIT_ENGINE,
         "IMAGE_EDIT_MODEL": request.app.state.config.IMAGE_EDIT_MODEL,
         "IMAGE_EDIT_SIZE": request.app.state.config.IMAGE_EDIT_SIZE,
@@ -365,6 +381,7 @@ def get_models(request: Request, user=Depends(get_verified_user)):
                 {"id": "dall-e-2", "name": "DALL·E 2"},
                 {"id": "dall-e-3", "name": "DALL·E 3"},
                 {"id": "gpt-image-1", "name": "GPT-IMAGE 1"},
+                {"id": "gpt-image-1.5", "name": "GPT-IMAGE 1.5"},
             ]
         elif request.app.state.config.IMAGE_GENERATION_ENGINE == "gemini":
             return [
@@ -495,15 +512,36 @@ def upload_image(request, image_data, content_type, metadata, user):
         process=False,
         user=user,
     )
+
+    if file_item and file_item.id:
+        # If chat_id and message_id are provided in metadata, link the file to the chat message
+        chat_id = metadata.get("chat_id")
+        message_id = metadata.get("message_id")
+
+        if chat_id and message_id:
+            Chats.insert_chat_files(
+                chat_id=chat_id,
+                message_id=message_id,
+                file_ids=[file_item.id],
+                user_id=user.id,
+            )
+
     url = request.app.url_path_for("get_file_content_by_id", id=file_item.id)
-    return url
+    return file_item, url
 
 
 @router.post("/generations")
+async def generate_images(
+    request: Request, form_data: CreateImageForm, user=Depends(get_verified_user)
+):
+    return await image_generations(request, form_data, user=user)
+
+
 async def image_generations(
     request: Request,
     form_data: CreateImageForm,
-    user=Depends(get_verified_user),
+    metadata: Optional[dict] = None,
+    user=None,
 ):
     # if IMAGE_SIZE = 'auto', default WidthxHeight to the 512x512 default
     # This is only relevant when the user has set IMAGE_SIZE to 'auto' with an
@@ -520,6 +558,9 @@ async def image_generations(
         size = form_data.size
 
     width, height = tuple(map(int, size.split("x")))
+
+    metadata = metadata or {}
+
     model = get_image_model(request)
 
     r = None
@@ -534,6 +575,10 @@ async def image_generations(
             if ENABLE_FORWARD_USER_INFO_HEADERS:
                 headers = include_user_info_headers(headers, user)
 
+            url = f"{request.app.state.config.IMAGES_OPENAI_API_BASE_URL}/images/generations"
+            if request.app.state.config.IMAGES_OPENAI_API_VERSION:
+                url = f"{url}?api-version={request.app.state.config.IMAGES_OPENAI_API_VERSION}"
+
             data = {
                 "model": model,
                 "prompt": form_data.prompt,
@@ -545,21 +590,22 @@ async def image_generations(
                 ),
                 **(
                     {}
-                    if "gpt-image-1" in request.app.state.config.IMAGE_GENERATION_MODEL
+                    if request.app.state.config.IMAGE_GENERATION_MODEL.startswith(
+                        "gpt-image"
+                    )
                     else {"response_format": "b64_json"}
                 ),
+                **(
+                    {}
+                    if not request.app.state.config.IMAGES_OPENAI_API_PARAMS
+                    else request.app.state.config.IMAGES_OPENAI_API_PARAMS
+                ),
             }
 
-            api_version_query_param = ""
-            if request.app.state.config.IMAGES_OPENAI_API_VERSION:
-                api_version_query_param = (
-                    f"?api-version={request.app.state.config.IMAGES_OPENAI_API_VERSION}"
-                )
-
             # Use asyncio.to_thread for the requests.post call
             r = await asyncio.to_thread(
                 requests.post,
-                url=f"{request.app.state.config.IMAGES_OPENAI_API_BASE_URL}/images/generations{api_version_query_param}",
+                url=url,
                 json=data,
                 headers=headers,
             )
@@ -575,7 +621,9 @@ async def image_generations(
                 else:
                     image_data, content_type = get_image_data(image["b64_json"])
 
-                url = upload_image(request, image_data, content_type, data, user)
+                _, url = upload_image(
+                    request, image_data, content_type, {**data, **metadata}, user
+                )
                 images.append({"url": url})
             return images
 
@@ -625,7 +673,9 @@ async def image_generations(
                     image_data, content_type = get_image_data(
                         image["bytesBase64Encoded"]
                     )
-                    url = upload_image(request, image_data, content_type, data, user)
+                    _, url = upload_image(
+                        request, image_data, content_type, {**data, **metadata}, user
+                    )
                     images.append({"url": url})
             elif model.endswith(":generateContent"):
                 for image in res["candidates"]:
@@ -634,8 +684,12 @@ async def image_generations(
                             image_data, content_type = get_image_data(
                                 part["inlineData"]["data"]
                             )
-                            url = upload_image(
-                                request, image_data, content_type, data, user
+                            _, url = upload_image(
+                                request,
+                                image_data,
+                                content_type,
+                                {**data, **metadata},
+                                user,
                             )
                             images.append({"url": url})
 
@@ -685,11 +739,11 @@ async def image_generations(
                     }
 
                 image_data, content_type = get_image_data(image["url"], headers)
-                url = upload_image(
+                _, url = upload_image(
                     request,
                     image_data,
                     content_type,
-                    form_data.model_dump(exclude_none=True),
+                    {**form_data.model_dump(exclude_none=True), **metadata},
                     user,
                 )
                 images.append({"url": url})
@@ -732,11 +786,11 @@ async def image_generations(
 
             for image in res["images"]:
                 image_data, content_type = get_image_data(image)
-                url = upload_image(
+                _, url = upload_image(
                     request,
                     image_data,
                     content_type,
-                    {**data, "info": res["info"]},
+                    {**data, "info": res["info"], **metadata},
                     user,
                 )
                 images.append({"url": url})
@@ -763,10 +817,13 @@ class EditImageForm(BaseModel):
 async def image_edits(
     request: Request,
     form_data: EditImageForm,
+    metadata: Optional[dict] = None,
     user=Depends(get_verified_user),
 ):
     size = None
     width, height = None, None
+    metadata = metadata or {}
+
     if (
         request.app.state.config.IMAGE_EDIT_SIZE
         and "x" in request.app.state.config.IMAGE_EDIT_SIZE
@@ -818,13 +875,13 @@ async def image_edits(
     except Exception as e:
         raise HTTPException(status_code=400, detail=ERROR_MESSAGES.DEFAULT(e))
 
-    def get_image_file_item(base64_string):
+    def get_image_file_item(base64_string, param_name="image"):
         data = base64_string
         header, encoded = data.split(",", 1)
         mime_type = header.split(";")[0].lstrip("data:")
         image_data = base64.b64decode(encoded)
         return (
-            "image",
+            param_name,
             (
                 f"{uuid.uuid4()}.png",
                 io.BytesIO(image_data),
@@ -849,7 +906,7 @@ async def image_edits(
                 **({"size": size} if size else {}),
                 **(
                     {}
-                    if "gpt-image-1" in request.app.state.config.IMAGE_EDIT_MODEL
+                    if request.app.state.config.IMAGE_EDIT_MODEL.startswith("gpt-image")
                     else {"response_format": "b64_json"}
                 ),
             }
@@ -859,7 +916,7 @@ async def image_edits(
                 files = [get_image_file_item(form_data.image)]
             elif isinstance(form_data.image, list):
                 for img in form_data.image:
-                    files.append(get_image_file_item(img))
+                    files.append(get_image_file_item(img, "image[]"))
 
             url_search_params = ""
             if request.app.state.config.IMAGES_EDIT_OPENAI_API_VERSION:
@@ -884,7 +941,9 @@ async def image_edits(
                 else:
                     image_data, content_type = get_image_data(image["b64_json"])
 
-                url = upload_image(request, image_data, content_type, data, user)
+                _, url = upload_image(
+                    request, image_data, content_type, {**data, **metadata}, user
+                )
                 images.append({"url": url})
             return images
 
@@ -937,8 +996,12 @@ async def image_edits(
                         image_data, content_type = get_image_data(
                             part["inlineData"]["data"]
                         )
-                        url = upload_image(
-                            request, image_data, content_type, data, user
+                        _, url = upload_image(
+                            request,
+                            image_data,
+                            content_type,
+                            {**data, **metadata},
+                            user,
                         )
                         images.append({"url": url})
 
@@ -1015,11 +1078,11 @@ async def image_edits(
                     }
 
                 image_data, content_type = get_image_data(image_url, headers)
-                url = upload_image(
+                _, url = upload_image(
                     request,
                     image_data,
                     content_type,
-                    form_data.model_dump(exclude_none=True),
+                    {**form_data.model_dump(exclude_none=True), **metadata},
                     user,
                 )
                 images.append({"url": url})
